package javax.management.monitor;
//java imports
//
import java.util.Date;
import java.util.Timer;
import java.util.TimerTask;

// jmx imports
//
import javax.management.ObjectName;
//import javax.management.MBeanNotificationInfo;
import javax.management.AttributeNotFoundException;
import javax.management.InstanceNotFoundException;
import javax.management.MBeanException;
import javax.management.ReflectionException;

/**
 * Defines a monitor MBean designed to observe periodically the values of an attribute 
 *
 * @version     0.1     
 */
public class PeriodicMonitor extends Monitor implements PeriodicMonitorMBean  {
    
    /**
     * Notification type denoting the observed attribute sample value.
     * This notification is only fired by periodic monitors.
     * <BR>The value of this notification type is <CODE>jmx.newMonitor.periodic</CODE>.
     */
    public static final String PERIODIC_SAMPLING = new String("jmx.monitor.periodic");

	/*
	 * ------------------------------------------ 
	 * PRIVATE VARIABLES
	 * ------------------------------------------
	 */
	/*private final static String[] types = {MonitorNotification.RUNTIME_ERROR,
			MonitorNotification.OBSERVED_OBJECT_ERROR,
			MonitorNotification.OBSERVED_ATTRIBUTE_ERROR,
			MonitorNotification.OBSERVED_ATTRIBUTE_TYPE_ERROR,
			PERIODIC_SAMPLING}; // !!!
	private final static MBeanNotificationInfo[] notifsInfo = {new MBeanNotificationInfo(
			types, "javax.management.monitor.MonitorNotification",
			"Notifications sent by the PeriodicMonitor MBean")};*/
	
	/**
	 * Derived gauges. <BR>
	 * Each element in this array corresponds to an observed object in the
	 * list.
	 */
	private transient Object derivedGauge[] = new Object[capacityIncrement];

	/**
	 * Derived gauge timestamps. <BR>
	 * Each element in this array corresponds to an observed object in the
	 * list.
	 */
	private transient long derivedGaugeTimestamp[] = new long[capacityIncrement];

	/**
	 * Timer.
	 */
	private transient Timer timer = null;

	/*
	 * ------------------------------------------ 
	 * PUBLIC METHODS
	 * ------------------------------------------
	 */

	/**
	 * Starts the periodic monitor.
	 */
	public synchronized void start() {
		if (!isActive()) {
			isActive = true;
			// Start the AlarmClock.
			timer = new Timer();
			timer.schedule(new PeriodicAlarmClock(this),
					getGranularityPeriod(), getGranularityPeriod());
		}
	}

	/**
	 * Stops the periodic monitor.
	 */
	/*
	 * This method is not synchronized, because if it were there could be a
	 * deadlock with a thread that attempted to get the lock on the monitor
	 * before being interrupted or noticing that it had
	 */
	public void stop() {
		if (isActive()) {
			// Stop the AlarmClock.
			if (timer != null) {
				timer.cancel();
				timer = null;
			}
			isActive = false;
		}
	}

	// GETTERS AND SETTERS
	//--------------------
	/**
	 * Sets the granularity period (in milliseconds). <BR>
	 * The default value of the granularity period is 10 seconds.
	 * 
	 * @param period
	 *            The granularity period value.
	 * @exception java.lang.IllegalArgumentException
	 *                The granularity period is less than or equal to zero.
	 * 
	 * @see NewMonitor#setGranularityPeriod(long)
	 */
	public synchronized void setGranularityPeriod(long period)
			throws java.lang.IllegalArgumentException {
		super.setGranularityPeriod(period);
		// Reschedule timer task if timer is already running
		//
		if (isActive()) {
			timer.cancel();
			timer = new Timer();
			timer.schedule(new PeriodicAlarmClock(this),
					getGranularityPeriod(), getGranularityPeriod());
		}
	}

	/**
	 * Gets the derived gauge of the specified object, if this object is
	 * contained in the set of observed MBeans, or <code>null</code>
	 * otherwise.
	 * 
	 * @param object
	 *            the name of the MBean whose derived gauge is required.
	 * 
	 * @return The derived gauge of the specified object.
	 * 
	 * @since JMX 1.2
	 */
	public Object getDerivedGauge(ObjectName object) {
		int index = indexOf(object);
		if (index != -1)
			return derivedGauge[index];
		else
			return null;
	}

	/**
	 * Gets the derived gauge timestamp of the specified object, if this object
	 * is contained in the set of observed MBeans, or <code>null</code>
	 * otherwise.
	 * 
	 * @param object
	 *            the name of the MBean whose derived gauge timestamp is
	 *            required.
	 * 
	 * @return The derived gauge timestamp of the specified object.
	 * 
	 * @since JMX 1.2
	 */
	public long getDerivedGaugeTimeStamp(ObjectName object) {
		int index = indexOf(object);
		if (index != -1)
			return derivedGaugeTimestamp[index];
		else
			return 0;
	}

	/*
	 * ------------------------------------------ 
	 * PRIVATE METHODS
	 * ------------------------------------------
	 */
	/**
	 * Updates the derived gauge and the derived gauge timestamp attributes of
	 * the observed object at the specified index.
	 * 
	 * @param scanObj
	 *            The value of the observed attribute.
	 * @param index
	 *            The index of the observed object.
	 */
	private void updateDerivedGauge(Object scanObj, int index) {
		derivedGaugeTimestamp[index] = (new Date()).getTime();
		derivedGauge[index] = scanObj;
	}

	/**
	 * Updates the notification attributes of the observed object at the
	 * specified index and notifies the listeners only once if the
	 * notifyMatch/notifyDiffer flag is set to <CODE>true</CODE>.
	 * 
	 * @param index
	 *            The index of the observed object.
	 */
	private void updateNotifications(int index) {
		sendNotification(
				PERIODIC_SAMPLING, 
				derivedGaugeTimestamp[index], "",
				derivedGauge[index], null, index);
	}

	/*
	 * ------------------------------------------ 
	 * PACKAGE METHODS
	 * ------------------------------------------
	 */
	/**
	 * This method is called by the periodic monitor each time the granularity
	 * period has been exceeded.
	 * 
	 * @param index
	 *            The index of the observed object.
	 */
	void notifyAlarmClock(int index) {
		Object scan_obj = null;
		String notif_type = null;
		try {
			if (isActive()) {
                
                // Check if the observed object and observed attribute are valid.
                //

                // Check that neither the observed object nor the observed attribute are null.
                // If the observed object or observed attribute is null, this means that
                // the monitor started before a complete initialization and nothing is done.
                //
                if ((getObservedObject(index) == null) || (getObservedAttribute() == null)) {
                    return;
                }
                
                // Check that the observed object is registered in the MBean server and
                // that the observed attribute belongs to the observed object.
                //
				try {
					scan_obj = server.getAttribute(getObservedObject(index),
							getObservedAttribute());
					if (scan_obj == null)
						return;
				} catch (NullPointerException np_ex) {
					if ((alreadyNotifieds[index] & RUNTIME_ERROR_NOTIFIED) != RESET_FLAGS_ALREADY_NOTIFIED) {
						return;
					} else {
						notif_type = MonitorNotification.RUNTIME_ERROR;
						setAlreadyNotified(index, RUNTIME_ERROR_NOTIFIED);
						throw new MonitorSettingException(
								"The periodic monitor must be registered in the MBean server.");
					}
				} catch (InstanceNotFoundException inf_ex) {
					if ((alreadyNotifieds[index] & OBSERVED_OBJECT_ERROR_NOTIFIED) != RESET_FLAGS_ALREADY_NOTIFIED) {
						return;
					} else {
						notif_type = MonitorNotification.OBSERVED_OBJECT_ERROR;
						setAlreadyNotified(index,
								OBSERVED_OBJECT_ERROR_NOTIFIED);
						throw new MonitorSettingException(
								"The observed object must be registered in the MBean server.");
					}
				} catch (AttributeNotFoundException anf_ex) {
					if ((alreadyNotifieds[index] & OBSERVED_ATTRIBUTE_ERROR_NOTIFIED) != RESET_FLAGS_ALREADY_NOTIFIED) {
						return;
					} else {
						notif_type = MonitorNotification.OBSERVED_ATTRIBUTE_ERROR;
						setAlreadyNotified(index,
								OBSERVED_ATTRIBUTE_ERROR_NOTIFIED);
						throw new MonitorSettingException(
								"The observed attribute must be accessible in the observed object.");
					}
				} catch (MBeanException mb_ex) {
					if ((alreadyNotifieds[index] & RUNTIME_ERROR_NOTIFIED) != RESET_FLAGS_ALREADY_NOTIFIED) {
						return;
					} else {
						notif_type = MonitorNotification.RUNTIME_ERROR;
						setAlreadyNotified(index, RUNTIME_ERROR_NOTIFIED);
						throw new MonitorSettingException(mb_ex.getMessage());
					}
				} catch (ReflectionException ref_ex) {
					if ((alreadyNotifieds[index] & OBSERVED_ATTRIBUTE_ERROR_NOTIFIED) != RESET_FLAGS_ALREADY_NOTIFIED) {
						return;
					} else {
						notif_type = MonitorNotification.OBSERVED_ATTRIBUTE_ERROR;
						setAlreadyNotified(index,
								OBSERVED_ATTRIBUTE_ERROR_NOTIFIED);
						throw new MonitorSettingException(ref_ex.getMessage());
					}
				}
				// Clear all already notified flags.
				resetAllAlreadyNotified(index);
				// Update the derived gauge attributes.
				updateDerivedGauge(scan_obj, index);
				// Notify the listeners.
				updateNotifications(index);
			}
		} catch (MonitorSettingException ms_ex) {
			// Create and send the monitor error notification.
			sendNotification(notif_type, derivedGaugeTimestamp[index], ms_ex
					.getMessage(), derivedGauge[index], null, index);
		}
	}

	/**
	 * This method is called when adding a new observed object in the vector.
	 * It updates all the specific arrays.
	 * 
	 * @param index
	 *            The index of the observed object.
	 */
	void insertSpecificElementAt(int index) {
		// Update derivedGauge, derivedGaugeTimestamp.
		insertObjectElementAt(derivedGauge, "", index);
		insertlongElementAt(derivedGaugeTimestamp, (new Date()).getTime(),
				index);
	}

	/**
	 * This method is called when removing an observed object from the vector.
	 * It updates all the specific arrays.
	 * 
	 * @param index
	 *            The index of the observed object.
	 */
	void removeSpecificElementAt(int index) {
		// Update derivedGauge, derivedGaugeTimestamp.
		removeObjectElementAt(derivedGauge, index);
		removelongElementAt(derivedGaugeTimestamp, index);
	}

	/**
	 * Inserts the specified value at the specified index in the specified
	 * Object array.
   * 
   * @param array the array to which the element must be added.
   * @param value the element that must be added.
   * @param index where the element must be added.
	 */
	void insertObjectElementAt(Object[] array, Object value, int index) { // !!!
		ensureObjectCapacity(array, elementCount + 1);
		System.arraycopy(array, index, array, index + 1, elementCount - index);
		array[index] = value;
	}

	/**
	 * Removes the component at the specified index from the specified Object
	 * array.
   * 
   * @param array the array from which the element must be removed.
   * @param index index of the element to be removed.
	 */
	void removeObjectElementAt(Object[] array, int index) {
		if (index < 0 || index >= elementCount)
			return;
		int j = elementCount - index - 1;
		if (j > 0) {
			System.arraycopy(array, index + 1, array, index, j);
		}
	}

	/**
	 * Increases the capacity of the specified Object array, if necessary, to
	 * ensure that it can hold at least the number of components specified by
	 * the minimum capacity argument.
   * 
   * @param array the array whose capacity must be increased, if necessary.
   * @param minCapacity the minimum size the array must have.
	 */
	void ensureObjectCapacity(Object[] array, int minCapacity) { // !!!
		int oldCapacity = array.length;
		// The array must be enlarged.
		//
		if (minCapacity > oldCapacity) {
			Object oldArray[] = array;
			int newCapacity = oldCapacity + capacityIncrement;
			if (newCapacity < minCapacity) {
				newCapacity = minCapacity;
			}
			array = new Number[newCapacity];
			System.arraycopy(oldArray, 0, array, 0, elementCount);
		}
	}

	/**
	 * PeriodicAlarmClock inner class: This class provides a simple
	 * implementation of an alarm clock MBean. The aim of this MBean is to set
	 * up an alarm which wakes up the string monitor every granularity period.
	 */
	private static class PeriodicAlarmClock extends TimerTask {
		PeriodicMonitor listener = null;
		/*
		 * ------------------------------------------ 
		 * CONSTRUCTORS
		 * ------------------------------------------
		 */
		public PeriodicAlarmClock(PeriodicMonitor listener) {
			this.listener = listener;
		}
		/*
		 * ------------------------------------------ 
		 * PUBLIC METHODS
		 * ------------------------------------------
		 */
		/**
		 * This method is called by the StringAlarmClock thread when it is
		 * started.
		 */
		public void run() {
			if (listener.isActive()) {
				for (int i = 0; i < listener.elementCount; i++) {
					listener.notifyAlarmClock(i);
				}
			}
		}
	}
}
