/***
 * Fractal JMX
 * Copyright (C) 2001-2002 France Telecom, INRIA
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: fractal@objectweb.org
 */

package org.objectweb.fractal.jmx.agent;

import javax.management.JMException;

/**
 * A simple interface for MBean manipulation on the Fractal JMX agent side. 
 * This interface allows an agent component implementing this interface to register MBeans in its scope, where:
 * <ul>
 * <li>the scope of the agent represents its super-components and all sub-components recursively enclosed in its 
 * super-components (or in itself if the agent has no super-components). 
 * <li>the MBeans represent component server interfaces or standard JMX 
 * {@link javax.management.monitor.MonitorMBean monitors} for observing component attributes. 
 * </ul>
 * 
 * <p>For example, in the architecture depicted below, where an agent is added to the composite B:
 * <center>
 * <p><img SRC="../../../../../../figures/fractal-jmx-scope.gif">
 * </center>
 * <UL>
 * <LI>The component B, C, D and Agent (itself) are in the agent scope and then candidate
 * to expose MBeans representing their server interfaces or observer on their attributes.
 * <LI>The component A is not in the agent scope, and then cannot be managed, since 
 * it is not a super-component of the agent.
 * </UL>
 * <br>
 * 
 * <p><B>Naming convention:</B> 
 * In JMX, an {@link javax.management.ObjectName ObjectName} represents a reference
 * to an MBean in the agent.
 * It is unique within an {@link javax.management.MBeanServer MBeanServer} and consists 
 * of two parts: the domain name and an unordered list of propert-value pairs.
 * The string representation of an object name must follow the following syntax:
 * <p><pre>
 *    [DomainName]:property=value[,property=value]*
 * </pre>
 * 
 * In Fractal JMX, the object name of an MBean representing a component server interface uses this syntax as follows: 
 * <p><pre>
 *    FC/[<i>path</i>]<i>component</i>@<i>id</i>[<i>shared</i>]:itf=<i>interface</i>
 * </pre>
 * where:
 * <UL>
 * <LI><I>component</I>: is the name of the component, as returned by {@link org.objectweb.fractal.api.control.NameController NameController}.
 * <LI><I>interface</I>: is the name of a server interface of the component,
 * as returned by {@link org.objectweb.fractal.api.type.InterfaceType InterfaceType}.
 * <LI><I>path</I>: is a (slash separated) name sequence of the successive super-components of the component, 
 * in the reverse order.
 * <LI><I>shared</I>: is an optional field set to "-shared" if the component may be identified by several <I>paths</I>
 * (i.e. if the component or some component, in its super-component hierarchy, is shared).
 * <LI><I>id</I>: is an inelegant, but unique, component identifier introduced to avoid JMX name clashes, 
 * since nothing prevents the same Fractal name sequence to identify several components.
 * </UL>
 *
 * For example, in the figure above, the MBean representing the interface I of the component D 
 * is identified by: 
 * 
 * <p><pre>
 *    FC/A/B/C/D@56e32:itf=I
 * </pre>
 * 
 * where @56e32 is the unique identifier of component D. 
 * This identifier is generated by the agent and can be ignored.
 * 
 * <p>This naming convention:
 * <ul>
 * <li>requires components to have a {@link org.objectweb.fractal.api.control.NameController NameController} 
 * and to avoid (in component or interface names) characters interpreted in specific ways by 
 * {@link javax.management.ObjectName ObjectName} (e.g. ":", "*", "&"...).
 * <li>is used by the 
 *  {@link AdminAttributes agent attributes}
 * to filter the relevant MBeans.  
 * Only the MBeans, whose 
 * {@link javax.management.ObjectName ObjectName}
 * matches a pattern defined by these attributes, are registered in the agent. 
 * These MBeans represent either component server interfaces or monitors for observing component attributes.
 * In the latter case, the naming convention used by the agent attributes considers some additional property-value pairs
 * reserved for the configuration of JMX monitors. 
 * </ul>
 * 
 * @version 0.1
 */
public interface Admin {

	/**
	 * Registers MBeans in the agent. 
	 * This method allows the agent component (implementing this interface) to register MBeans in its scope, 
	 * where:
	 * <ul>
	 * <li>the scope of the agent represents its super-components and all sub-components recursively enclosed in its 
	 * super-components (or in itself if the agent has no super-components). 
	 * <li>the MBeans represent component (server) interfaces or standard JMX 
	 * {@link javax.management.monitor.MonitorMBean monitors} for observing component attributes. 
	 * </ul>
	 * 
	 * <P>This method: 
	 * <UL>
	 * <LI>first unregisters (previously registered) MBeans in the agent.</LI> 
	 * <LI>then registers MBeans in the scope that match the {@link AdminAttributes filter attributes} of the agent. 
	 * </UL>
	 * 
	 * @throws JMException if a problem occurs during the MBean registration.
	 */
	void expose() throws JMException;

}
