/***
 * Fractal JMX
 * Copyright (C) 2003 France Telecom R&D
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: fractal@objectweb.org
 */
package org.objectweb.fractal.jmx.comm;

import org.objectweb.fractal.api.control.BindingController;
import org.objectweb.fractal.api.control.LifeCycleController;

import org.objectweb.fractal.jmx.agent.AdminAttributes;

// JMX
import javax.management.ObjectName;
import javax.management.JMException;
import javax.management.InstanceNotFoundException;
import javax.management.JMRuntimeException;
// JMX-RI
import com.sun.jdmk.comm.HtmlAdaptorServer;

/**
 * A component based on the HTML protocol adaptor provided by the JMX<SUP><FONT SIZE=-2>TM</FONT></SUP> 1.2 
 * Reference Implementation. 
 * This adaptor allows an HTML browser to manage all MBeans in the Fractal JMX agent side. 
 * To connect a browser to the agent, open this page in a web browser:
 * <UL>
 * </CODE>http://host:port</CODE>
 * </UL>
 * where:
 * <UL>
 * <LI><I>host</I> is the host name of the machine on which the agent is running (or localhost if the agent is local).
 * <LI><I>port</I> is the port number used by the HTML server in the agent (default: 8082).
 * </UL>
 * This HTML protocol adaptor provides the following main HTML pages for managing MBeans in the agent:
 * <UL>
 * <LI><I>Agent View</I>: Provides a list of object names of all the MBeans registered in the agent.
 * <LI><I>Agent Administration</I>: Registers and unregisters MBeans in the agent.
 * <LI><I>MBean View</I>: Reads and writes MBean attributes and perform operations on MBeans in the agent.
 * </UL>
 * 
 * <p>The binding controller part of this component allows to set the (required) agent component.
 * The {@link CommunicatorAttributes attribute controller} part allows to set the <I>port</I> number used to 
 * connect a browser to the agent.
 * The life-cycle controller part registers this component in the agent when it starts, 
 * if it is not already registered or if its <I>port</I> attribute has changed. 
 * The underlying HTML protocol adaptor is implemented as a dynamic MBean,
 * and can be managed:
 * <ul>
 * <li>as described in the JMX<SUP><FONT SIZE=-2>TM</FONT></SUP> 1.2 Reference Implementation documentation.
 * <br>It is identified in the agent by the {@link ObjectName}: RI_HtmlAdaptor:type=html,port=<I>port</I>
 * </li> 
 * <li> or indirectly by accessing the MBean representing the {@link CommunicatorAttributes attribute controller} 
 * of this component.
 * <br>In this case, it is required to stop and (re)start the component to take account a new <I>port</I> number,
 * and to browse a page with the new <I>port</I>.
 * </li> 
 * </ul>
 * 
 * 
 * @version 0.1
 */
public class RI_HtmlAdaptor implements BindingController, LifeCycleController, CommunicatorAttributes {
	private static final String ADAPTOR = "RI_HtmlAdaptor";
	private AdminAttributes _adminAtt;
	private HtmlAdaptorServer _htmlAdaptor = new HtmlAdaptorServer();
	private int _port = _htmlAdaptor.getPort();

	// --------------------------------------------------------------------------
	// Implementation of the BindingController interface
	// --------------------------------------------------------------------------
	public String[] listFc() {
		return new String[] { "adminAtt" };
	}

	public Object lookupFc(final String itfName) {
		if (itfName.equals("adminAtt"))
			return _adminAtt;
		else
			return null;
	}

	public void bindFc(final String itfName, final Object itfValue) {
		if (itfName.equals("adminAtt")) {
			_adminAtt = (AdminAttributes) itfValue;
		}
	}

	public void unbindFc(final String itfName) {
		if (itfName.equals("adminAtt")) {
			_adminAtt = null;
		}
	}

	// -------------------------------------------------------------------------
	// implementation of the LifeCycleController interface
	// -------------------------------------------------------------------------
	public String getFcState() {
		return null;
	}

	public synchronized void startFc() {
		ObjectName oName;
		String id = '@' + Integer.toHexString(this.hashCode());
		try {
			//System.out.println("->tries to start with port=" + port);
			if (_htmlAdaptor != null
				&& _htmlAdaptor.getState() == HtmlAdaptorServer.ONLINE
				&& _htmlAdaptor.getPort() == _port) {
				//System.out.println("<-already started with port=" + port);
				return;
			}
			// unregister & stops
			if (_htmlAdaptor != null) {
				_htmlAdaptor.stop();
				oName = new ObjectName(ADAPTOR + id + ":type=html,port=" + _htmlAdaptor.getPort());
				try {
					_adminAtt.getRawMBeanServer().unregisterMBean(oName);
				} catch (InstanceNotFoundException ignore) {}
			}
			// registers & starts
			_htmlAdaptor = new HtmlAdaptorServer(_port);
			oName = new ObjectName(ADAPTOR + id + ":type=html,port=" + _port);
			_adminAtt.getRawMBeanServer().registerMBean(_htmlAdaptor, oName);
			_htmlAdaptor.start();
			// checks
			if (!_htmlAdaptor.waitState(HtmlAdaptorServer.ONLINE, 5000))
				throw new JMRuntimeException(
					"HtmlAdaptorServer failed to start with port="
						+ _htmlAdaptor.getPort()
						+ ", current state="
						+ _htmlAdaptor.getState());
			//System.out.println("<-started with port=" + htmlAdaptor.getPort());
		} catch (JMException e) {
			throw new JMRuntimeException(e.toString());
		}
	}

	public void stopFc() {}

	// -------------------------------------------------------------------------
	// implements Attribute controller
	// -------------------------------------------------------------------------
	public int getPort() {
		return _port;
	}

	public void setPort(final int port) {
		this._port = port;
	}
}
